==> prepare-raspbian-sd-v1.3.0.tar.gz: prepare-raspbian-sd/apis/install-api/install-api.sh <==
#!/bin/bash -e
#
# install-api:install-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t exit_if_variable_not_define_or_empty )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required Function exit_if_variable_not_define_or_empty not found.' >&2
  exit 1
fi

function install_if_needed {
  local -r skip_update="$1"
  local    pkg_manager="$2" # R/W
  shift
  shift

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.' >&2
    exit 1
  fi

  if [ "${skip_update}" != 'skip_update' ] ; then
    "${pkg_manager}" update  || exit 1
  fi

  local command_name=
  local package_name=

  while [ $# -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || exit 1
    fi
  done
}

function install_set_chmod_for_list {
  local -r mode="$1"
  local -r files="$2"
  local    file=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      chmod -v "${mode}" "${INSTALL_DATA_DIR}/${file}" || exit 1
    fi
  done <<<"${files}"
}

function install_links {
  local -r files="$1"
  local   file=
  local   realfile=
  local   linkfile=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ ! -f "${realfile}" ] ; then
        echofatal "'${realfile}' not found." >&2
        exit 1
      fi

      if [ -L "${linkfile}" ] ; then
        if [ "$( realpath "${linkfile}" )" = "${realfile}" ] ; then
          rm -v "${linkfile}"
        fi
      fi
      ln -sv "${realfile}" "${linkfile}" || exit 1
    fi
  done <<<"${files}"
}

function install_main {
  mkdir -pv "${INSTALL_DATA_DIR}" || exit 1
  rsync -av \
    --exclude='package' \
    --exclude='.git' \
    --exclude='.gitignore' \
    --exclude='.gitlab-ci.yml' \
    --exclude='samples' \
    "${PROJECT_HOME}/"* "${INSTALL_DATA_DIR}" || exit 1
  chown -Rv root:root "${INSTALL_DATA_DIR}" || exit 1

  find "${INSTALL_DATA_DIR}" -type f -exec chmod -v 444 {} \; || exit 1
  find "${INSTALL_DATA_DIR}" -type d -exec chmod -v 755 {} \; || exit 1

  install_set_chmod_for_list 555 "${INSTALL_EXE}"
  install_links "${INSTALL_LINKS}"

  echoinfo "$( realpath "$0" ) done." >&2
}

function exit_if_not_root {
  if [ "${EUID}" -ne 0 ] ; then
    echo 'This script must be run as root.' >&2
    exit 1
  fi
}

function remove_previous_version_before_install {
  local remove='false'

  if [ -f "${REMOVE_EXE}" ] ; then
    bash "${REMOVE_EXE}"
    remove='true'
  elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
    if [ -f "${REMOVE_OLD}" ] ; then
      bash "${REMOVE_OLD}"
      remove='true'
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    echowarn "File not found: '${REMOVE_EXE}'" >&2
  fi
}

function remove_links {
  local -r files="$1"
  local  file=
  local  realfile=
  local  linkfile=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ -L "${linkfile}" ] ; then
        if [ "$( readlink "${linkfile}" )" = "${realfile}" ] || [ -f "$( readlink "${linkfile}" )" ] ; then
          rm -v "${linkfile}"
        fi
      fi
    fi
  done <<<"${files}"
}

function remove_main {
  exit_if_not_root

  local scriptpath=

  scriptpath="$( realpath "$0" )"

  local remove='false'

  if [ "${REMOVE_EXE}" != "${scriptpath}" ] &&  [ -f "${REMOVE_EXE}" ] ; then
    bash "${REMOVE_EXE}"
    remove='true'
  elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
    if [ -f "${REMOVE_OLD}" ] ; then
      bash "${REMOVE_OLD}"
      remove='true'
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    remove_links "${INSTALL_LINKS}"

    rm -fvr "${INSTALL_DATA_DIR}" || exit 1
  fi

  echoinfo "${scriptpath} done." >&2
}

exit_if_variable_not_define_or_empty 'INSTALL_BIN_DIR'
exit_if_variable_not_define_or_empty 'INSTALL_DATA_DIR'
exit_if_variable_not_define_or_empty 'REMOVE_EXE'

==> prepare-raspbian-sd-v1.3.2.tar.gz: prepare-raspbian-sd/apis/install-api/install-api.sh <==
#!/bin/bash -e
#
# install-api:install-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t exit_if_variable_not_define_or_empty )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required Function exit_if_variable_not_define_or_empty not found.' >&2
  exit 1
fi

function _get_package_manager_ {
  local pkg_manager="$1" # R/W

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.' >&2
    exit 1
  fi
  echo "${pkg_manager}"
}

function install_update {
  local pkg_manager=

  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    exit 1
  fi

  "${pkg_manager}" update || exit 1
}

function install_install_if_needed {
  local pkg_manager=

  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    exit 1
  fi
  shift

  local command_name=
  local package_name=

  while [ $# -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || exit 1
    fi
  done
}

function install_if_needed { # DEPRECATED
  local -r skip_update="$1"
  local    pkg_manager="$2" # R/W
  shift
  shift

  echofatal 'install_if_needed is deprecated, use install_update and install_install_if_needed instead'

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.' >&2
    exit 1
  fi

  if [ "${skip_update}" != 'skip_update' ] ; then
    "${pkg_manager}" update  || exit 1
  fi

  local command_name=
  local package_name=

  while [ $# -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || exit 1
    fi
  done
}

function install_set_chmod_for_list {
  local -r mode="$1"
  local -r files="$2"
  local    file=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      chmod -v "${mode}" "${INSTALL_DATA_DIR}/${file}" || exit 1
    fi
  done <<<"${files}"
}

function install_links {
  local -r files="$1"
  local   file=
  local   realfile=
  local   linkfile=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ ! -f "${realfile}" ] ; then
        echofatal "'${realfile}' not found." >&2
        exit 1
      fi

      if [ -L "${linkfile}" ] ; then
        if [ "$( realpath "${linkfile}" )" = "${realfile}" ] ; then
          rm -v "${linkfile}"
        fi
      fi
      ln -sv "${realfile}" "${linkfile}" || exit 1
    fi
  done <<<"${files}"
}

function install_main {
  mkdir -pv "${INSTALL_DATA_DIR}" || exit 1
  rsync -av \
    --exclude='package' \
    --exclude='.git' \
    --exclude='.gitignore' \
    --exclude='.gitlab-ci.yml' \
    --exclude='samples' \
    "${PROJECT_HOME}/"* "${INSTALL_DATA_DIR}" || exit 1
  chown -Rv root:root "${INSTALL_DATA_DIR}" || exit 1

  find "${INSTALL_DATA_DIR}" -type f -exec chmod -v 444 {} \; || exit 1
  find "${INSTALL_DATA_DIR}" -type d -exec chmod -v 755 {} \; || exit 1

  install_set_chmod_for_list 555 "${INSTALL_EXE}"
  install_links "${INSTALL_LINKS}"

  echoinfo "$( realpath "$0" ) done." >&2
}

function exit_if_not_root {
  if [ "${EUID}" -ne 0 ] ; then
    echo 'This script must be run as root.' >&2
    exit 1
  fi
}

function remove_previous_version_before_install {
  local remove='false'

  if [ -f "${REMOVE_EXE}" ] ; then
    bash "${REMOVE_EXE}"
    remove='true'
  elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
    if [ -f "${REMOVE_OLD}" ] ; then
      bash "${REMOVE_OLD}"
      remove='true'
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    echowarn "File not found: '${REMOVE_EXE}'" >&2
  fi
}

function remove_links {
  local -r files="$1"
  local  file=
  local  realfile=
  local  linkfile=

  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ -L "${linkfile}" ] ; then
        if [ "$( readlink "${linkfile}" )" = "${realfile}" ] || [ -f "$( readlink "${linkfile}" )" ] ; then
          rm -v "${linkfile}"
        fi
      fi
    fi
  done <<<"${files}"
}

function remove_main {
  exit_if_not_root

  local scriptpath=

  scriptpath="$( realpath "$0" )"

  local remove='false'

  if [ "${REMOVE_EXE}" != "${scriptpath}" ] &&  [ -f "${REMOVE_EXE}" ] ; then
    bash "${REMOVE_EXE}"
    remove='true'
  elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
    if [ -f "${REMOVE_OLD}" ] ; then
      bash "${REMOVE_OLD}"
      remove='true'
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    remove_links "${INSTALL_LINKS}"

    rm -fvr "${INSTALL_DATA_DIR}" || exit 1
  fi

  echoinfo "${scriptpath} done." >&2
}

exit_if_variable_not_define_or_empty 'INSTALL_BIN_DIR'
exit_if_variable_not_define_or_empty 'INSTALL_DATA_DIR'
exit_if_variable_not_define_or_empty 'REMOVE_EXE'
